package com.hzya.frame.nifiapi.operation.impl;

import com.hzya.frame.nifiapi.model.joincreateoracleapp.CreateAppInstanceJoin;
import com.hzya.frame.nifiapi.model.joincreateprocessorandupdateparam.CreateProcessorAndUpdateParamJoin;
import com.hzya.frame.nifiapi.model.joincreparamcontext.CreateParamContextJoin;
import com.hzya.frame.nifiapi.model.joinfindneedmodifycontroller.FindNeedModifyController;
import com.hzya.frame.nifiapi.model.joingetallcontrollerservice.ControllerServices19;
import com.hzya.frame.nifiapi.model.joingetcontroller.Component12;
import com.hzya.frame.nifiapi.model.joingetcontroller.ControllerService12;
import com.hzya.frame.nifiapi.model.resultcreateoracleapp.CreateAppInstanceResult;
import com.hzya.frame.nifiapi.model.resultprocessorsinfo.Component16;
import com.hzya.frame.nifiapi.model.resultprocessorsinfo.ProcessorsInfo16;
import com.hzya.frame.nifiapi.model.resultprocessorsinfo.Revision16;
import com.hzya.frame.nifiapi.operation.INifiBaseOperation;
import com.hzya.frame.nifiapi.operation.INifiAppOperation;
import com.hzya.frame.nifiapi.service.NifiApiService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * NiFi 操作实现类，封装通用操作
 *
 * @Author：liuyang
 * @Package：com.hzya.frame.nifiapi.operation.impl
 * @Project：fw-nifi
 * @name：NifiOperationImpl
 * @Date：2025/5/21 17:06
 * @Filename：NifiOperationImpl
 */
@Repository(value = "NifiOperationImpl")
public class NifiAppOperationImpl implements INifiAppOperation {

    Logger logger = LoggerFactory.getLogger(NifiAppOperationImpl.class);

    @Autowired
    private NifiApiService nifiApiService;

    @Autowired
    private INifiBaseOperation baseOperation;

    @Override
    public CreateAppInstanceResult createAppContext(CreateAppInstanceJoin createOracleApp) throws Exception {
        CreateAppInstanceResult createAppInstanceResult = new CreateAppInstanceResult();
        try {
            //1.实例化应用
            String newProcessGroupId = null;
            String appProcessGroupId = createOracleApp.getAppProcessGroupId();
            String parentProcessGroupId = createOracleApp.getParentProcessGroupId();
            String copyTargetProcessGroupId = createOracleApp.getCopyTargetProcessGroupId();
            if (appProcessGroupId != null && parentProcessGroupId != null && copyTargetProcessGroupId != null) {
                newProcessGroupId = baseOperation.instantiateApp(appProcessGroupId, parentProcessGroupId, copyTargetProcessGroupId);
                logger.info("实例化后的流程组id:{}", newProcessGroupId);
            }
            //2.新增应用对应的上下文参数
            CreateParamContextJoin createParamContextJoin = createOracleApp.getCreateParamContextJoin();
            if (createParamContextJoin != null) {
                createParamContextJoin.setProcessGroupsId(newProcessGroupId);
                baseOperation.createParameterContextsBindingProcessGroup(createParamContextJoin);
            }
            //3.修改控制器，默认激活当前流程组下的所有控制器
            FindNeedModifyController findNeedModifyController = createOracleApp.getFindNeedModifyController();
            if (findNeedModifyController != null) {
                List<ControllerService12> needModifyController = findNeedModifyController.getNeedModifyController();
                for (ControllerService12 controllerService12 : needModifyController) {
//                    controllerService12.setId();

                    //根据修改标识，查找控制器
                    String modifyMarkId = controllerService12.getModifyMarkId();
                    ControllerServices19 needModifyControllerServicesByProcessGroup = baseOperation.getNeedModifyControllerServicesByProcessGroup(newProcessGroupId, modifyMarkId);

                    controllerService12.setId(needModifyControllerServicesByProcessGroup.getId());

                    Component12 component = controllerService12.getComponent();
                    component.setId(needModifyControllerServicesByProcessGroup.getId());
                }

                findNeedModifyController.setProcessGroupId(newProcessGroupId);
                baseOperation.findNeedModifyControllerAndChangesOccur(findNeedModifyController, true, true);
            }

            createAppInstanceResult.setNewProcessGroupId(newProcessGroupId);
        } catch (Exception e) {
            throw new Exception(e);
        }
        return createAppInstanceResult;
    }

    @Override
    public String createProcessorAndUpdateParam(CreateProcessorAndUpdateParamJoin createProcessorAndUpdateParamJoin) throws Exception {
        String newProcessId = null;
        try {
            //拷贝处理器
            String targetProcessorId = createProcessorAndUpdateParamJoin.getTargetProcessorId();
            String parentProcessGroupId = createProcessorAndUpdateParamJoin.getParentProcessGroupId();
            String copyTargetProcessGroupId = createProcessorAndUpdateParamJoin.getCopyTargetProcessGroupId();
            //允许不修改！
            Component16 component = createProcessorAndUpdateParamJoin.getComponent();

            newProcessId = baseOperation.copyProcessor(targetProcessorId, parentProcessGroupId, copyTargetProcessGroupId);
            if (newProcessId != null && component != null) {
                //更新处理器
                ProcessorsInfo16 processor = nifiApiService.getProcessor(newProcessId);
                Revision16 revision16 = processor.getRevision();
                String version = revision16.getVersion();

                Revision16 revision161 = new Revision16();
                revision161.setVersion(version);

                component.setId(newProcessId);

                ProcessorsInfo16 processorsInfo16 = new ProcessorsInfo16();
                processorsInfo16.setRevision(revision161);
                processorsInfo16.setComponent(component);
                ProcessorsInfo16 processor2 = nifiApiService.updateProcessor(newProcessId, processorsInfo16);
                String id = processor2.getId();
                logger.info("新处理器id:{}" + id);
            }
        } catch (Exception e) {
            throw new Exception(e);
        }
        return newProcessId;
    }

//    @Override
//    public void createMysqlApp(CreateMysqlApp createMysqlApp) throws Exception {
//        try {
//            //1.实例化Mysql应用
//            String appProcessGroupId = createMysqlApp.getAppProcessGroupId();
//            String appProcessGroupId1 = createMysqlApp.getParentProcessGroupId();
//            String copyTargetProcessGroupId = createMysqlApp.getCopyTargetProcessGroupId();
//            String processId = baseOperation.instantiateApp(appProcessGroupId, appProcessGroupId1, copyTargetProcessGroupId);
//            logger.info("Mysql应用实例化后的流程组id:{}", processId);
//        } catch (Exception e) {
//            throw new Exception(e);
//        }
//    }
}